@@ -9,18 +9,20 @@ __all__ = [
     "SessionAPI",
 ]
 
+import typing
 from collections import (
     Iterable,
     namedtuple,
 )
 import json
+from urllib.parse import urlparse
 
 import aiohttp
 
 from . import helpers
 from .. import utils
 from ..utils import profiles
-from ..utils.async import asynchronous
+from ..utils.maas_async import asynchronous
 
 
 class SessionError(Exception):
@@ -43,6 +45,7 @@ class SessionAPI:
             raise SessionError(str(error))
         else:
             session = cls(description, credentials)
+            session.scheme = urlparse(url).scheme
             session.insecure = insecure
             return session
 
@@ -52,7 +55,10 @@ class SessionAPI:
 
         :see: `ProfileStore`.
         """
-        return cls(profile.description, profile.credentials)
+        session = cls(profile.description, profile.credentials)
+        session.scheme = urlparse(profile.url).scheme
+        session.insecure = profile.other.get('insecure', False)
+        return session
 
     @classmethod
     def fromProfileName(cls, name):
@@ -76,6 +82,7 @@ class SessionAPI:
         profile = await helpers.login(
             url=url, username=username, password=password, insecure=insecure)
         session = cls(profile.description, profile.credentials)
+        session.scheme = urlparse(url).scheme
         session.insecure = insecure
         return profile, session
 
@@ -92,10 +99,12 @@ class SessionAPI:
         profile = await helpers.connect(
             url=url, apikey=apikey, insecure=insecure)
         session = cls(profile.description, profile.credentials)
+        session.scheme = urlparse(url).scheme
         session.insecure = insecure
         return profile, session
 
     # Set these on instances.
+    scheme = 'http'
     insecure = False
     debug = False
 
@@ -296,6 +305,11 @@ class ActionAPI:
         """
         data = dict(data)
         params = {name: data.pop(name) for name in self.handler.params}
+        for key, value in data.copy().items():
+            if isinstance(value, typing.Mapping):
+                del data[key]
+                for nested_key, nested_value in value.items():
+                    data[key + '_' + nested_key] = nested_value
         for key, value in data.items():
             if key.startswith('_'):
                 data[key[1:]] = data.pop(key)
@@ -371,7 +385,10 @@ class CallAPI:
         # TODO: this is el-cheapo URI Template
         # <http://tools.ietf.org/html/rfc6570> support; use uritemplate-py
         # <https://github.com/uri-templates/uritemplate-py> here?
-        return self.action.handler.uri.format(**self.__params)
+        uri = urlparse(self.action.handler.uri)
+        if uri.scheme != self.action.handler.session.scheme:
+            uri = uri._replace(scheme=self.action.handler.session.scheme)
+        return uri.geturl().format(**self.__params)
 
     def rebind(self, **params):
         """Rebind the parameters into the URI.
