@@ -8,6 +8,7 @@ __all__ = [
 import asyncio
 import base64
 import bson
+import json
 from http import HTTPStatus
 import typing
 
@@ -27,6 +28,7 @@ from .nodes import (
     NodesType,
     NodeTypeMeta,
 )
+from .pods import Pod
 from .subnets import Subnet
 from .zones import Zone
 from ..bones import CallError
@@ -68,9 +70,9 @@ class MachinesType(NodesType):
     async def create(
             cls, architecture: str, mac_addresses: typing.Sequence[str],
             power_type: str,
-            power_parameters: typing.Mapping[str, typing.Any]=None, *,
-            subarchitecture: str=None, min_hwe_kernel: str=None,
-            hostname: str=None, domain: typing.Union[int, str]=None):
+            power_parameters: typing.Mapping[str, typing.Any] = None, *,
+            subarchitecture: str = None, min_hwe_kernel: str = None,
+            hostname: str = None, domain: typing.Union[int, str] = None):
         """
         Create a new machine.
 
@@ -98,7 +100,8 @@ class MachinesType(NodesType):
             "power_type": power_type,
         }
         if power_parameters is not None:
-            params["power_parameters"] = power_parameters
+            params["power_parameters"] = json.dumps(
+                power_parameters, sort_keys=True)
         if subarchitecture is not None:
             params["subarchitecture"] = subarchitecture
         if min_hwe_kernel is not None:
@@ -111,25 +114,27 @@ class MachinesType(NodesType):
 
     async def allocate(
             cls, *,
-            hostname: str=None,
-            architectures: typing.Sequence[str]=None,
-            cpus: int=None,
-            fabrics: typing.Sequence[FabricParam]=None,
-            interfaces: typing.Sequence[InterfaceParam]=None,
-            memory: float=None,
-            pod: str=None,
-            pod_type: str=None,
-            storage: typing.Sequence[str]=None,
-            subnets: typing.Sequence[SubnetParam]=None,
-            tags: typing.Sequence[str]=None,
-            zone: typing.Union[str, Zone]=None,
-            not_fabrics: typing.Sequence[FabricParam]=None,
-            not_subnets: typing.Sequence[SubnetParam]=None,
-            not_tags: typing.Sequence[str]=None,
-            not_zones: typing.Sequence[ZoneParam]=None,
-            agent_name: str=None, comment: str=None,
-            bridge_all: bool=None, bridge_stp: bool=None, bridge_fd: int=None,
-            dry_run: bool=None, verbose: bool=None):
+            hostname: str = None,
+            architectures: typing.Sequence[str] = None,
+            cpus: int = None,
+            fabrics: typing.Sequence[FabricParam] = None,
+            interfaces: typing.Sequence[InterfaceParam] = None,
+            memory: float = None,
+            pod: typing.Union[str, Pod] = None,
+            not_pod: typing.Union[str, Pod] = None,
+            pod_type: str = None,
+            not_pod_type: str = None,
+            storage: typing.Sequence[str] = None,
+            subnets: typing.Sequence[SubnetParam] = None,
+            tags: typing.Sequence[str] = None,
+            zone: typing.Union[str, Zone] = None,
+            not_fabrics: typing.Sequence[FabricParam] = None,
+            not_subnets: typing.Sequence[SubnetParam] = None,
+            not_tags: typing.Sequence[str] = None,
+            not_zones: typing.Sequence[ZoneParam] = None,
+            agent_name: str = None, comment: str = None,
+            bridge_all: bool = None, bridge_stp: bool = None,
+            bridge_fd: int = None, dry_run: bool = None, verbose: bool = None):
         """
         Allocate a machine.
 
@@ -147,8 +152,12 @@ class MachinesType(NodesType):
         :type memory: `int`
         :param pod: The pod to allocate the machine from.
         :type pod: `str`
+        :param not_pod: Pod the machine must not be located in.
+        :type not_pod: `str`
         :param pod_type: The type of pod to allocate the machine from.
         :type pod_type: `str`
+        :param not_pod_type: Pod type the machine must not be located in.
+        :type not_pod_type: `str`
         :param subnets: The subnet(s) the desired machine must be linked to.
         :type subnets: sequence of `str` or `int` or `Subnet`
         :param storage: The storage contraint to match.
@@ -191,8 +200,8 @@ class MachinesType(NodesType):
             'arch': architectures,
             'cpu_count': str(cpus) if cpus else None,
             'mem': str(memory) if memory else None,
-            'pod': pod,
             'pod_type': pod_type,
+            'not_pod_type': not_pod_type,
             'storage': storage,
             'tags': tags,
             'not_tags': not_tags,
@@ -214,6 +223,23 @@ class MachinesType(NodesType):
                 get_param_arg('interfaces', idx, Interface, nic)
                 for idx, nic in enumerate(interfaces)
             ]
+        if pod is not None:
+            if isinstance(pod, Pod):
+                params["pod"] = pod.name
+            elif isinstance(pod, str):
+                params["pod"] = pod
+            else:
+                raise TypeError(
+                    "pod must be a str or Pod, not %s" % type(pod).__name__)
+        if not_pod is not None:
+            if isinstance(not_pod, Pod):
+                params["not_pod"] = not_pod.name
+            elif isinstance(not_pod, str):
+                params["not_pod"] = not_pod
+            else:
+                raise TypeError(
+                    "not_pod must be a str or Pod, not %s" %
+                    type(not_pod).__name__)
         if subnets is not None:
             params["subnets"] = [
                 get_param_arg('subnets', idx, Subnet, subnet)
@@ -320,6 +346,14 @@ class Machine(Node, metaclass=MachineType):
         "architecture", check_optional(str), check_optional(str))
     boot_disk = ObjectFieldRelated(
         "boot_disk", "BlockDevice", readonly=True)
+    boot_interface = ObjectFieldRelated(
+        "boot_interface", "Interface", readonly=True)
+    block_devices = ObjectFieldRelatedSet(
+        "blockdevice_set", "BlockDevices", reverse=None)
+    bcaches = ObjectFieldRelatedSet(
+        "bcaches", "Bcaches", reverse=None)
+    cache_sets = ObjectFieldRelatedSet(
+        "cache_sets", "BcacheCacheSets", reverse=None)
     cpus = ObjectField.Checked(
         "cpu_count", check(int), check(int))
     disable_ipv4 = ObjectField.Checked(
@@ -338,17 +372,6 @@ class Machine(Node, metaclass=MachineType):
         "osystem", check(str), readonly=True)
     owner_data = ObjectField.Checked(
         "owner_data", check(dict), check(dict))
-
-    boot_interface = ObjectFieldRelated(
-        "boot_interface", "Interface", readonly=True)
-    block_devices = ObjectFieldRelatedSet(
-        "blockdevice_set", "BlockDevices", reverse=None)
-
-    power_state = ObjectField.Checked(
-        "power_state", to(PowerState), readonly=True)
-    power_type = ObjectField.Checked(
-        "power_type", check(str), readonly=True)
-
     status = ObjectField.Checked(
         "status", to(NodeStatus), readonly=True)
     status_action = ObjectField.Checked(
@@ -357,18 +380,11 @@ class Machine(Node, metaclass=MachineType):
         "status_message", check_optional(str), readonly=True)
     status_name = ObjectField.Checked(
         "status_name", check(str), readonly=True)
-
-    bcaches = ObjectFieldRelatedSet(
-        "bcaches", "Bcaches", reverse=None)
-    cache_sets = ObjectFieldRelatedSet(
-        "cache_sets", "BcacheCacheSets", reverse=None)
     raids = ObjectFieldRelatedSet(
         "raids", "Raids", reverse=None)
     volume_groups = ObjectFieldRelatedSet(
         "volume_groups", "VolumeGroups", reverse=None)
 
-    # swap_size
-
     async def save(self):
         """Save the machine in MAAS."""
         orig_owner_data = self._orig_data['owner_data']
@@ -381,12 +397,7 @@ class Machine(Node, metaclass=MachineType):
             await self._handler.set_owner_data(**params_diff)
             self._data['owner_data'] = self._data['owner_data']
 
-    async def get_power_parameters(self):
-        """Get the power paramters for this machine."""
-        data = await self._handler.power_parameters(system_id=self.system_id)
-        return data
-
-    async def abort(self, *, comment: str=None):
+    async def abort(self, *, comment: str = None):
         """Abort the current action.
 
         :param comment: Reason for aborting the action.
@@ -397,21 +408,21 @@ class Machine(Node, metaclass=MachineType):
         }
         if comment:
             params["comment"] = comment
-        self._data = await self._handler.abort(**params)
+        self._reset(await self._handler.abort(**params))
         return self
 
     async def clear_default_gateways(self):
         """Clear default gateways."""
-        self._data = await self._handler.clear_default_gateways(
-            system_id=self.system_id)
+        self._reset(await self._handler.clear_default_gateways(
+            system_id=self.system_id))
         return self
 
     async def commission(
-            self, *, enable_ssh: bool=None, skip_networking: bool=None,
-            skip_storage: bool=None,
-            commissioning_scripts: typing.Sequence[str]=None,
-            testing_scripts: typing.Sequence[str]=None,
-            wait: bool=False, wait_interval: int=5):
+            self, *, enable_ssh: bool = None, skip_networking: bool = None,
+            skip_storage: bool = None,
+            commissioning_scripts: typing.Sequence[str] = None,
+            testing_scripts: typing.Sequence[str] = None,
+            wait: bool = False, wait_interval: int = 5):
         """Commission this machine.
 
         :param enable_ssh: Prevent the machine from powering off after running
@@ -447,11 +458,11 @@ class Machine(Node, metaclass=MachineType):
                 len(commissioning_scripts) > 0):
             params["commissioning_scripts"] = ",".join(commissioning_scripts)
         if testing_scripts is not None:
-            if len(testing_scripts) == 0:
-                params["testing_scripts"] = ""
+            if len(testing_scripts) == 0 or testing_scripts == "none":
+                params["testing_scripts"] = ["none"]
             else:
                 params["testing_scripts"] = ",".join(testing_scripts)
-        self._data = await self._handler.commission(**params)
+        self._reset(await self._handler.commission(**params))
         if not wait:
             return self
         else:
@@ -459,7 +470,7 @@ class Machine(Node, metaclass=MachineType):
             while self.status in [
                     NodeStatus.COMMISSIONING, NodeStatus.TESTING]:
                 await asyncio.sleep(wait_interval)
-                self._data = await self._handler.read(system_id=self.system_id)
+                self._reset(await self._handler.read(system_id=self.system_id))
             if self.status == NodeStatus.FAILED_COMMISSIONING:
                 msg = "{hostname} failed to commission.".format(
                     hostname=self.hostname)
@@ -471,9 +482,9 @@ class Machine(Node, metaclass=MachineType):
             return self
 
     async def deploy(
-            self, *, user_data: typing.Union[bytes, str]=None,
-            distro_series: str=None, hwe_kernel: str=None, comment: str=None,
-            wait: bool=False, wait_interval: int=5):
+            self, *, user_data: typing.Union[bytes, str] = None,
+            distro_series: str = None, hwe_kernel: str = None,
+            comment: str = None, wait: bool = False, wait_interval: int = 5):
         """Deploy this machine.
 
         :param user_data: User-data to provide to the machine when booting. If
@@ -501,14 +512,14 @@ class Machine(Node, metaclass=MachineType):
             params["hwe_kernel"] = hwe_kernel
         if comment is not None:
             params["comment"] = comment
-        self._data = await self._handler.deploy(**params)
+        self._reset(await self._handler.deploy(**params))
         if not wait:
             return self
         else:
             # Wait for the machine to be fully deployed
             while self.status == NodeStatus.DEPLOYING:
                 await asyncio.sleep(wait_interval)
-                self._data = await self._handler.read(system_id=self.system_id)
+                self._reset(await self._handler.read(system_id=self.system_id))
             if self.status == NodeStatus.FAILED_DEPLOYMENT:
                 msg = "{hostname} failed to deploy.".format(
                     hostname=self.hostname
@@ -516,7 +527,8 @@ class Machine(Node, metaclass=MachineType):
                 raise FailedDeployment(msg, self)
             return self
 
-    async def enter_rescue_mode(self, wait: bool=False, wait_interval: int=5):
+    async def enter_rescue_mode(
+            self, wait: bool = False, wait_interval: int = 5):
         """
         Send this machine into 'rescue mode'.
 
@@ -524,8 +536,8 @@ class Machine(Node, metaclass=MachineType):
         :param wait_interval: How often to poll, defaults to 5 seconds
         """
         try:
-            self._data = await self._handler.rescue_mode(
-                system_id=self.system_id)
+            self._reset(await self._handler.rescue_mode(
+                system_id=self.system_id))
         except CallError as error:
             if error.status == HTTPStatus.FORBIDDEN:
                 message = "Not allowed to enter rescue mode"
@@ -539,7 +551,7 @@ class Machine(Node, metaclass=MachineType):
             # Wait for machine to finish entering rescue mode
             while self.status == NodeStatus.ENTERING_RESCUE_MODE:
                 await asyncio.sleep(wait)
-                self._data = await self._handler.read(system_id=self.system_id)
+                self._reset(await self._handler.read(system_id=self.system_id))
             if self.status == NodeStatus.FAILED_ENTERING_RESCUE_MODE:
                 msg = "{hostname} failed to enter rescue mode.".format(
                     hostname=self.hostname
@@ -547,7 +559,8 @@ class Machine(Node, metaclass=MachineType):
                 raise RescueModeFailure(msg, self)
             return self
 
-    async def exit_rescue_mode(self, wait: bool=False, wait_interval: int=5):
+    async def exit_rescue_mode(
+            self, wait: bool = False, wait_interval: int = 5):
         """
         Exit rescue mode.
 
@@ -555,9 +568,9 @@ class Machine(Node, metaclass=MachineType):
         :param wait_interval: How often to poll, defaults to 5 seconds
         """
         try:
-            self._data = await self._handler.exit_rescue_mode(
+            self._reset(await self._handler.exit_rescue_mode(
                 system_id=self.system_id
-            )
+            ))
         except CallError as error:
             if error.status == HTTPStatus.FORBIDDEN:
                 message = "Not allowed to exit rescue mode."
@@ -570,7 +583,7 @@ class Machine(Node, metaclass=MachineType):
             # Wait for machine to finish exiting rescue mode
             while self.status == NodeStatus.EXITING_RESCUE_MODE:
                 await asyncio.sleep(wait_interval)
-                self._data = await self._handler.read(system_id=self.system_id)
+                self._reset(await self._handler.read(system_id=self.system_id))
             if self.status == NodeStatus.FAILED_EXITING_RESCUE_MODE:
                 msg = "{hostname} failed to exit rescue mode.".format(
                     hostname=self.hostname
@@ -594,7 +607,7 @@ class Machine(Node, metaclass=MachineType):
         data = await self._handler.details(system_id=self.system_id)
         return bson.decode_all(data)[0]
 
-    async def mark_broken(self, *, comment: str=None):
+    async def mark_broken(self, *, comment: str = None):
         """Mark broken.
 
         :param comment: Reason machine is broken.
@@ -605,10 +618,10 @@ class Machine(Node, metaclass=MachineType):
         }
         if comment:
             params["comment"] = comment
-        self._data = await self._handler.mark_broken(**params)
+        self._reset(await self._handler.mark_broken(**params))
         return self
 
-    async def mark_fixed(self, *, comment: str=None):
+    async def mark_fixed(self, *, comment: str = None):
         """Mark fixes.
 
         :param comment: Reason machine is fixed.
@@ -619,13 +632,13 @@ class Machine(Node, metaclass=MachineType):
         }
         if comment:
             params["comment"] = comment
-        self._data = await self._handler.mark_fixed(**params)
+        self._reset(await self._handler.mark_fixed(**params))
         return self
 
     async def release(
-            self, *, comment: str=None, erase: bool=None,
-            secure_erase: bool=None, quick_erase: bool=None,
-            wait: bool=False, wait_interval: int=5):
+            self, *, comment: str = None, erase: bool = None,
+            secure_erase: bool = None, quick_erase: bool = None,
+            wait: bool = False, wait_interval: int = 5):
         """
         Release the machine.
 
@@ -649,7 +662,7 @@ class Machine(Node, metaclass=MachineType):
             "secure_erase": secure_erase,
             "quick_erase": quick_erase,
         })
-        self._data = await self._handler.release(**params)
+        self._reset(await self._handler.release(**params))
         if not wait:
             return self
         else:
@@ -658,8 +671,8 @@ class Machine(Node, metaclass=MachineType):
                     NodeStatus.RELEASING, NodeStatus.DISK_ERASING]:
                 await asyncio.sleep(wait_interval)
                 try:
-                    self._data = await self._handler.read(
-                        system_id=self.system_id)
+                    self._reset(await self._handler.read(
+                        system_id=self.system_id))
                 except CallError as error:
                     if error.status == HTTPStatus.NOT_FOUND:
                         # Release must have been on a machine in a pod. This
@@ -681,7 +694,8 @@ class Machine(Node, metaclass=MachineType):
             return self
 
     async def power_on(
-            self, comment: str=None, wait: bool=False, wait_interval: int=5):
+            self, comment: str = None,
+            wait: bool = False, wait_interval: int = 5):
         """
         Power on.
 
@@ -696,7 +710,7 @@ class Machine(Node, metaclass=MachineType):
         if comment is not None:
             params["comment"] = comment
         try:
-            self._data = await self._handler.power_on(**params)
+            self._reset(await self._handler.power_on(**params))
         except CallError as error:
             if error.status == HTTPStatus.FORBIDDEN:
                 message = "Not allowed to power on machine."
@@ -711,7 +725,7 @@ class Machine(Node, metaclass=MachineType):
             # Wait for machine to be powered on.
             while self.power_state == PowerState.OFF:
                 await asyncio.sleep(wait_interval)
-                self._data = await self._handler.read(system_id=self.system_id)
+                self._reset(await self._handler.read(system_id=self.system_id))
             if self.power_state == PowerState.ERROR:
                 msg = "{hostname} failed to power on.".format(
                     hostname=self.hostname
@@ -720,8 +734,8 @@ class Machine(Node, metaclass=MachineType):
             return self
 
     async def power_off(
-            self, stop_mode: PowerStopMode=PowerStopMode.HARD,
-            comment: str=None, wait: bool=False, wait_interval: int=5):
+            self, stop_mode: PowerStopMode = PowerStopMode.HARD,
+            comment: str = None, wait: bool = False, wait_interval: int = 5):
         """
         Power off.
 
@@ -738,7 +752,7 @@ class Machine(Node, metaclass=MachineType):
         if comment is not None:
             params["comment"] = comment
         try:
-            self._data = await self._handler.power_off(**params)
+            self._reset(await self._handler.power_off(**params))
         except CallError as error:
             if error.status == HTTPStatus.FORBIDDEN:
                 message = "Not allowed to power off machine."
@@ -753,7 +767,7 @@ class Machine(Node, metaclass=MachineType):
             # Wait for machine to be powered off.
             while self.power_state == PowerState.ON:
                 await asyncio.sleep(wait_interval)
-                self._data = await self._handler.read(system_id=self.system_id)
+                self._reset(await self._handler.read(system_id=self.system_id))
             if self.power_state == PowerState.ERROR:
                 msg = "{hostname} failed to power off.".format(
                     hostname=self.hostname
@@ -780,19 +794,47 @@ class Machine(Node, metaclass=MachineType):
         """
         Restore machine's configuration to its initial state.
         """
-        self._data = await self._handler.restore_default_configuration(
-            system_id=self.system_id)
+        self._reset(await self._handler.restore_default_configuration(
+            system_id=self.system_id))
 
     async def restore_networking_configuration(self):
         """
         Restore machine's networking configuration to its initial state.
         """
-        self._data = await self._handler.restore_networking_configuration(
-            system_id=self.system_id)
+        self._reset(await self._handler.restore_networking_configuration(
+            system_id=self.system_id))
 
     async def restore_storage_configuration(self):
         """
         Restore machine's storage configuration to its initial state.
         """
-        self._data = await self._handler.restore_storage_configuration(
-            system_id=self.system_id)
+        self._reset(await self._handler.restore_storage_configuration(
+            system_id=self.system_id))
+
+    async def lock(self, *, comment: str = None):
+        """Lock the machine to prevent changes.
+
+        :param comment: Reason machine was locked.
+        :type comment: `str`
+        """
+        params = {
+            "system_id": self.system_id
+        }
+        if comment:
+            params["comment"] = comment
+        self._reset(await self._handler.lock(**params))
+        return self
+
+    async def unlock(self, *, comment: str = None):
+        """Unlock the machine allowing changes.
+
+        :param comment: Reason machine was unlocked.
+        :type comment: `str`
+        """
+        params = {
+            "system_id": self.system_id
+        }
+        if comment:
+            params["comment"] = comment
+        self._reset(await self._handler.unlock(**params))
+        return self
