@@ -52,7 +52,7 @@ class Vlan(Object, metaclass=VlanType):
         "id", check(int), readonly=True)
 
     fabric = ObjectFieldRelated(
-        "fabric_id", "Fabric", readonly=True, pk=0)
+        "fabric_id", "Fabric", pk=0)
     vid = ObjectField.Checked(
         "vid", check(int), check(int), pk=1)
 
@@ -64,7 +64,7 @@ class Vlan(Object, metaclass=VlanType):
         "space", "Space",
         map_func=lambda _, data: {'name': data, '__incomplete__': True})
 
-    relay_vlan = ObjectFieldRelated("relay_vlan", "Vlan")
+    relay_vlan = ObjectFieldRelated("relay_vlan", "Vlan", use_data_setter=True)
     dhcp_on = ObjectField.Checked(
         "dhcp_on", check(bool), check(bool))
     primary_rack = ObjectFieldRelated("primary_rack", "RackController")
@@ -77,6 +77,56 @@ class Vlan(Object, metaclass=VlanType):
         return super(Vlan, self).__repr__(
             fields={"name", "vid"})
 
+    async def save(self):
+        # Store fabric_id so its reset on save. `fabric_id` is not returned
+        # by the API, this is stored through `local_data`.
+        fabric_id = self._data['fabric_id']
+        if ('fabric_id' in self._changed_data and
+                self._changed_data['fabric_id'] is not None):
+            if (self._orig_data['fabric_id'] and
+                    self._changed_data['fabric_id'] == (
+                        self._orig_data['fabric_id'])):
+                # Fabric didn't really change, the object was just set to the
+                # same Fabric
+                del self._changed_data['fabric_id']
+            else:
+                # Allow the update of the fabric by setting the 'fabric' field
+                # with the new 'fabric_id'.
+                self._changed_data['fabric'] = self._changed_data['fabric_id']
+
+        if ('relay_vlan' in self._changed_data and
+                self._changed_data['relay_vlan']):
+            # Update uses the ID of the VLAN, not the VLAN object.
+            self._changed_data['relay_vlan'] = (
+                self._changed_data['relay_vlan']['id'])
+            if ('relay_vlay' in self._orig_data and
+                    self._orig_data['relay_vlan'] and
+                    'id' in self._orig_data['relay_vlan'] and
+                    self._changed_data['relay_vlan'] == (
+                        self._orig_data['relay_vlan']['id'])):
+                # VLAN didn't really change, the object was just set to the
+                # same VLAN.
+                del self._changed_data['relay_vlan']
+            else:
+                self._changed_data['relay_vlan'] = (
+                    self._changed_data['relay_vlan'])
+
+        # Don't call `super().save()` because `vid` can be changed and it has
+        # to be handled specially because its in the resource_uri path.
+        if self._changed_data:
+            update_data = dict(self._changed_data)
+            update_data.update({
+                key: self._orig_data[key]
+                for key in self._handler.params
+            })
+            update_data['vid'] = update_data['_vid'] = self._orig_data['vid']
+            if 'vid' in self._changed_data:
+                update_data['_vid'] = self._changed_data['vid']
+            data = await self._handler.update(**update_data)
+            # Set fabric_id because it was lost from the `update`.
+            data['fabric_id'] = fabric_id
+            self._reset(data)
+
     async def delete(self):
         """Delete this VLAN."""
         # Since the VID can be changed for the VLAN, we always use the vid
@@ -111,11 +161,11 @@ class VlansType(ObjectType):
 
     async def create(
             cls, fabric: Union[Fabric, int], vid: int, *,
-            name: str=None, description: str=None, mtu: int=None,
-            relay_vlan: Union[Vlan, int]=None, dhcp_on: bool=False,
-            primary_rack: Union[RackController, str]=None,
-            secondary_rack: Union[RackController, str]=None,
-            space: Union[Space, int]=None):
+            name: str = None, description: str = None, mtu: int = None,
+            relay_vlan: Union[Vlan, int] = None, dhcp_on: bool = False,
+            primary_rack: Union[RackController, str] = None,
+            secondary_rack: Union[RackController, str] = None,
+            space: Union[Space, int] = None):
         """
         Create a `Vlan` in MAAS.
 
